@@ -448,8 +448,8 @@ s100:"' . mce_escape( __('100%') ) . '",
 s110:"' . mce_escape( __('110%') ) . '",
 s120:"' . mce_escape( __('120%') ) . '",
 s130:"' . mce_escape( __('130%') ) . '",
-img_title:"' . mce_escape( __('Edit Image Title') ) . '",
-caption:"' . mce_escape( __('Edit Image Caption') ) . '",
-alt:"' . mce_escape( __('Edit Alternate Text') ) . '"
+img_title:"' . mce_escape( __('Title') ) . '",
+caption:"' . mce_escape( __('Caption') ) . '",
+alt:"' . mce_escape( __('Alternate Text') ) . '"
 });
 ';
